/*
 *	General functions.
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <math.h>

#include "hr_general.h"
#include "hr_defines.h"
#include "hr_bobs.h"
#include "hr_fonts.h"
#include "hr_bitmaps.h"
#include "hr_mouse.h"
#include "hr_keyboard.h"
#include "hr_zones.h"
#include "hr_buttons.h"
#include "hr_vconsole.h"
#include "hr_windows.h"


struct hr_screen {
	int flags;
	int mode;
	GraphicsContext *logical;
	GraphicsContext *temp;
	unsigned char *mask;
	unsigned char *palette_store;
	struct hr_window *firstwindow;
	struct hr_window *active_window;
	struct hr_menu *rootmenu;
};


/* "Closes" a screen (deallocs mem used by it) */
void hr_screen_close(struct hr_screen *screen)
{
	if(screen!=NULL){
		if(screen->flags & SCREEN_VIRTUAL){
			if((screen->logical)!=NULL)
			  gl_freecontext(screen->logical);
			if((screen->temp)!=NULL)
			  gl_freecontext(screen->temp);
		}
		/* FIX! Close windows belonging to this screen.*/
		/* ADD */
		free(screen->palette_store);
		free(screen->mask);
		free(screen);
	}
}


/* Opens a screen, and recalcs/sets mouse position if necessary.
 * Does not print any error messages. */
struct hr_screen *hr_screen_open(int mode,int flags)
{
	int ow,oh,do_switch=1;
	struct hr_screen *screen;
	hr.default_button=NULL;
	
	/* Is it necessary to change screenmode? */
	if(hr.active_screen!=NULL){
	  if(hr.current_screenmode==mode)
		  do_switch=0;
	}
	else
	  hr_screen_close(hr.active_screen);/* FIX! FIX! FIX! FIX! REMOVE! */
#ifdef NO_SCREENMODE_SWITCHING
	/* Never change screenmode after the initial mode switch.*/
	if(hr.current_screenmode==TEXT && mode!=TEXT)
	  do_switch=1;
	else
	  do_switch=0;
#else
#ifdef SWITCH_KLUDGE
	if(do_switch){
		if((hr.current_screenmode!=TEXT)&&
			(mode!=TEXT)){
			/* Hmmm... if screenmode is changed from, say, 320x200 to 640x480,
			 * ctrl-c just hangs the program.
			 * This temporary change to TEXT mode is necessary on my machine.*/
			usleep(SWITCH_KLUDGE);
			vga_setmode(TEXT);
			usleep(SWITCH_KLUDGE);
		}
	}
#endif
#endif
	/* Change to correct gfx mode.*/
	if (!vga_hasmode(mode))
	  return NULL;
	else {
		hr.active_screen=screen=malloc(sizeof(struct hr_screen));
		ow=WIDTH;oh=HEIGHT;
		/* Make sure pointer is ready to be re-initialized */
		hr_update_mouse(NONE,BOB_REMOVE);
		/* Set mode, gl context and enable clipping.*/
		if(do_switch){
			hr.current_screenmode=mode;
			vga_setmode(mode);
			gl_setclippingwindow(0,0,WIDTH-1,HEIGHT-1);
		}
		else
		  gl_clearscreen(0);					/* FIX. */
		gl_setcontextvga(mode);
		if(flags & SCREEN_NOCLIP)
		  gl_disableclipping();
		else
		  gl_enableclipping();
		/**/
		screen->flags=flags;
		screen->mode=mode;
		/* set physical gfx context */
		gl_getcontext(hr.physicalscreen);
		/* set logical gfx context if required.*/
		if(flags & SCREEN_VIRTUAL){
			/* Virtual gfx context */
			gl_setcontextvgavirtual(mode);
			if(flags & SCREEN_NOCLIP)
			  gl_disableclipping();
			else
			  gl_enableclipping();
			screen->logical = gl_allocatecontext();
			gl_getcontext(screen->logical);
		}
		/* Set some pointers */
		screen->temp=NULL;
		screen->mask=NULL;
		screen->palette_store=NULL;
		if(flags & MOUSE_CENTER)
		  mouse_setposition(WIDTH/2,HEIGHT/2);
		/* Root window */
		screen->active_window=screen->firstwindow=NULL;
		/* Root menu */
		screen->rootmenu=NULL;
		/* Reposition mouse pointer */
		if(((WIDTH!=ow) || (HEIGHT!=oh)) && (ow>0) && (oh>0)) 
		  mouse_setposition((hr_mouse.x*WIDTH)/ow,(hr_mouse.y*HEIGHT)/oh);
		/* Update new coordinates in mouse structure */
		/* This should only be done when necessary. FIX! */
		hr_update_mouse(NONE,BOB_NOIMG);
		return screen;
	}
}


/* Changes active screen, sets screenmode if necessary and so on.*/
void hr_set_active_screen(struct hr_screen *screen)
{
	/**/
	return;
}


/**/
void hr_update_screen(struct hr_screen *screen,int flags)
{
	/**/
	return;
}
