/*
 *	Tykkipeli 2
 *	Game options screen.
 */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include "defines.h"
#include "tp2.h"
#include "game_screen.h"
#include "players.h"
#include "misc.h"
#include "info_windows.h"
#include "option_screen_defines.h"
#include "init_exit.h"

#include "hr_defines.h"
#include "hr_mouse.h"
#include "hr_bobs.h"
#include "hr_buttons.h"
#include "hr_fonts.h"
#include "hr_general.h"
#include "hr_bitmaps.h"
#include "hr_keyboard.h"
#include "hr_input_handler.h"
#include "hr_screens.h"
#include "hr_graphics.h"


/* These are used for the starfield behind the logo.*/
#define OPTION_STAR_N 112

#define MAX_OPTIONBUTTONS 44
struct hr_button **optionbutton;
struct hr_button **namebutton;
struct hr_button **infobutton;

struct option_star{
	int x;	
	int y;
	int xs;
	int ys;
	int color;
};

struct option_star *option_stars;


/* Init the top logo effect. */
void init_options_effect()
{
	int s;
	/* Star colors.*/
	for(s=0;s<64;s++)
	  gl_setpalettecolor(192+s,s,s,s);
	/* Scroller color.*/
	gl_setpalettecolor(62,63,0,63);
	/* Get logo palette */
	hr_use_bitmap_palette(logo,128,192,NONE);
	gfx_counter=scrolltext_counter=0;
	gl_fillbox(0,0,320,120,192);
	gl_putboxmask(160-(logo->width)/2,56-(logo->height)/2,
					  logo->width,logo->height,logo->data);
	option_stars=malloc(OPTION_STAR_N*sizeof(struct option_star));
	for(s=0;s<OPTION_STAR_N;s++) {
		(option_stars+s)->x=random()%320;
		(option_stars+s)->y=random()%112;
		(option_stars+s)->xs=10+random()%4;
		(option_stars+s)->ys=1+random()%3;
		(option_stars+s)->color=210+random()%30;
	}

}


/* Update logo effect. Re-inits mouse pointer if necessary.*/
int options_effect()
{
   int s,a,x=hr_mouse.x,y=hr_mouse.y;
	if((x<320) && (y<120))
	  hr_update_mouse(NONE,BOB_REMOVE);
	/* Calc s, update scroller */
	s=(++gfx_counter)%8;
	if(s==0) {
		if(*(instructions+(++scrolltext_counter))==0)
		  scrolltext_counter=0;
	}
	vga_waitretrace();
	gl_copybox(1,112,319,8,0,112);
	hr_partial_write(318,112,62,0,*(instructions+scrolltext_counter),
								s,0,s,7,hr.font8x8,NONE);
	/* Stars */
	a=gfx_counter%360;
	for(s=0;s<OPTION_STAR_N;s++)  {
		/* Remove old star */
		if(vga_getpixel((option_stars+s)->x,(option_stars+s)->y)>192)
		  gl_setpixel((option_stars+s)->x,(option_stars+s)->y,192);
		/* Calc new X position */
		(option_stars+s)->x=((option_stars+s)->x+
									 (((option_stars+s)->xs*pcos[a])>>SCALE));
		if((option_stars+s)->x<0)
		  ((option_stars+s)->x)+=320;
		if((option_stars+s)->x>319)
		  (option_stars+s)->x-=320;
		/* Calc new Y position */
		(option_stars+s)->y=((option_stars+s)->y+(option_stars+s)->ys);
		if((option_stars+s)->y>111)
		  ((option_stars+s)->y)-=112;
		/* Draw the dot */
		if(vga_getpixel((option_stars+s)->x,(option_stars+s)->y)==192)
		  gl_setpixel((option_stars+s)->x,
						  (option_stars+s)->y,(option_stars+s)->color);
	}
	if((x<320) && (y<120))
	  hr_update_mouse(NONE,BOB_INIT);
	return 0;
}


/* Creates the option screen */
void create_option_screen()
{
	int r;
	/* Roster */
	hr_bevelbox(320,0,591,479,hr.def_colors,NONE);
	hr_bevelbox(592,0,639,479,hr.def_colors,NONE);
	/* malloc tables for button pointers */
	namebutton=malloc(ROSTER_SIZE*sizeof(struct hr_button *));
	infobutton=malloc(ROSTER_SIZE*sizeof(struct hr_button *));
	optionbutton=malloc(MAX_OPTIONBUTTONS*sizeof(struct hr_button *));
	/* Put 2 strings in buffer[].*/
	sprintf(buffer,"-");
	for(r=0;r<ROSTER_SIZE;r++)  {
		/* Create the wide button with player's name on it */
		namebutton[r]=hr_create_button(322,2+r*14,589,14+r*14,
												 hr.font8x8,hr.def_colors,
												 roster[r].name,NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
		/* Print the ratio numbers */
		if(roster[r].games>0)
		  sprintf(buffer+4,"%d",magic_ratio(roster[r].deaths,
														roster[r].kills,
														roster[r].suicides));
		/* Create smaller button that opens info window */
		infobutton[r]=hr_create_button(594,2+r*14,637,14+r*14,
												 hr.font8x8,hr.def_colors,
												 buffer+((roster[r].games>0)?(4):(0)),
												 "Info",0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
		/* Add help text */
		hr_update_button(infobutton[r],UPDATE_HELPTEXT,"Player info");
	}
	/* Nullify option button table.*/
	for(r=0;r<MAX_OPTIONBUTTONS;r++)
	  optionbutton[r]=NULL;
	/* game options */
	/* Rounds */
  	hr_bevelbox(0,120,179,137,hr.def_colors,NONE);
	hr_write(4,126,1,0,"Rounds",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[0]=hr_create_button(86,122,99,135,
												hr.font8x8,hr.def_colors,
												"-",NULL,0,
												BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[1]=hr_create_button(164,122,177,135,
												hr.font8x8,hr.def_colors,
												"+",NULL,0,
												BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	
	/* Money */
	hr_bevelbox(0,138,179,185,hr.def_colors,NONE);
	hr_write(4,142,1,0,"Money",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[2]=hr_create_button(86,140,99,153,
												hr.font8x8,hr.def_colors,
												"-",NULL,0,
												BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[3]=hr_create_button(164,140,177,153,
												hr.font8x8,hr.def_colors,
												"+",NULL,0,
												BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	
	hr_write(4,157,1,0,"Winning $",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[4]=hr_create_button(86,155,99,168,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[5]=hr_create_button(164,155,177,168,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	
   hr_write(4,172,1,0,"Interest",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[6]=hr_create_button(86,170,99,183,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[7]=hr_create_button(164,170,177,183,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	
	/* game options */
  	hr_bevelbox(0,186,179,365,hr.def_colors,NONE);
	hr_write(4,190,1,0,"Gravity",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[8]=hr_create_button(86,188,99,201,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[9]=hr_create_button(164,188,177,201,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	
	hr_write(4,205,1,0,"Wind",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[10]=hr_create_button(86,203,99,216,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[11]=hr_create_button(164,203,177,216,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	
	hr_write(4,220,1,0,"Edges",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[12]=hr_create_button(86,218,99,231,
												 hr.font8x8,hr.def_colors,
												 "<",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[13]=hr_create_button(164,218,177,231,
												 hr.font8x8,hr.def_colors,
												 ">",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);	
	
	hr_write(4,235,1,0,"Steepness",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[14]=hr_create_button(86,233,99,246,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[15]=hr_create_button(164,233,177,246,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);	

	hr_write(4,250,1,0,"Air Rstnce",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[16]=hr_create_button(86,248,99,261,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);
	optionbutton[17]=hr_create_button(164,248,177,261,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,
												 BUTTON_ENABLED|BUTTON_NO_TAB_CYCLE);	

	/* Computer player settings */
	hr_bevelbox(0,366,179,398,hr.def_colors,NONE);
	hr_write(4,370,1,0,"Droids",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[18]=hr_create_button(86,368,99,381,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,BUTTON_NO_TAB_CYCLE);
	optionbutton[19]=hr_create_button(164,368,177,381,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,BUTTON_NO_TAB_CYCLE);
	
	hr_write(4,385,1,0,"Droid IQ",hr.font8x8,STRING_START,STRING_END,NONE);
	optionbutton[20]=hr_create_button(86,383,99,396,
												 hr.font8x8,hr.def_colors,
												 "-",NULL,0,BUTTON_NO_TAB_CYCLE);
	optionbutton[21]=hr_create_button(164,383,177,396,
												 hr.font8x8,hr.def_colors,
												 "+",NULL,0,BUTTON_NO_TAB_CYCLE);
	
	/* On/off options */
	hr_bevelbox(180,120,319,398,hr.def_colors,NONE);
	optionbutton[22]=hr_create_button(182,122,317,133,
												 hr.font8x8,hr.def_colors,
												 "Tracers",NULL,0,BUTTON_ENABLED);
	hr_update_button(optionbutton[22],UPDATE_HELPTEXT,
						  "Warheads leave trails on the screen?");
	optionbutton[23]=hr_create_button(182,135,317,146,
												 hr.font8x8,hr.def_colors,
												 "Disasters",NULL,0,NONE);
	hr_update_button(optionbutton[23],UPDATE_HELPTEXT,
						  "Both natural and unnatural disasters occur sometimes?");
	optionbutton[24]=hr_create_button(182,148,317,159,
												 hr.font8x8,hr.def_colors,
												 "Particles",NULL,0,BUTTON_ENABLED);
	hr_update_button(optionbutton[24],UPDATE_HELPTEXT,
						  "Explosions throw dirt up the air?");
	optionbutton[25]=hr_create_button(182,161,317,172,
												 hr.font8x8,hr.def_colors,
												 "Sound Effects",NULL,0,NONE);
	hr_update_button(optionbutton[25],UPDATE_HELPTEXT,
						  "Sound effects.");
	optionbutton[26]=hr_create_button(182,174,317,185,
												 hr.font8x8,hr.def_colors,
												 "Unsafe storage",NULL,0,NONE);
	hr_update_button(optionbutton[26],UPDATE_HELPTEXT,
						  "Chance of accidental launching of weapons when you die?");
   optionbutton[27]=hr_create_button(182,187,317,198,
												 hr.font8x8,hr.def_colors,
												 "Suicide Penalty",NULL,0,NONE);
	hr_update_button(optionbutton[27],UPDATE_HELPTEXT,
						  "Committing suicide is bad for your purse?");
   optionbutton[28]=hr_create_button(182,200,317,211,
												 hr.font8x8,hr.def_colors,
												 "AimAid(tm)",NULL,0,NONE);
	hr_update_button(optionbutton[28],UPDATE_HELPTEXT,
						  "Draw the AimAid(tm) crosshairs?");
   optionbutton[29]=hr_create_button(182,213,317,224,
												 hr.font8x8,hr.def_colors,
												 "Alien terrain",NULL,0,NONE);
	hr_update_button(optionbutton[29],UPDATE_HELPTEXT,
						  "Enable ~ireally~i weird terrain palettes?");
   optionbutton[30]=hr_create_button(182,226,317,237,
												 hr.font8x8,hr.def_colors,
												 "Alien sky",NULL,0,NONE);
	hr_update_button(optionbutton[30],UPDATE_HELPTEXT,
						  "Enable weird sky color slides.");
   optionbutton[31]=hr_create_button(182,239,317,250,
												 hr.font8x8,hr.def_colors,
												 "Random order",NULL,0,NONE);
	hr_update_button(optionbutton[31],UPDATE_HELPTEXT,
						  "Players make their selections in random order?");
   optionbutton[32]=hr_create_button(182,252,317,263,
												 hr.font8x8,hr.def_colors,
												 "GroundSmoothing",NULL,0,NONE);
	hr_update_button(optionbutton[32],UPDATE_HELPTEXT,
						  "Make terrain look smoother after explosions?");
   optionbutton[33]=hr_create_button(182,265,317,276,
												 hr.font8x8,hr.def_colors,
												 "Terrain pixmaps",NULL,0,NONE);
	hr_update_button(optionbutton[33],UPDATE_HELPTEXT,
						  "Use precalced images instead of random terrains?");
   optionbutton[34]=hr_create_button(182,278,317,289,
												 hr.font8x8,hr.def_colors,
												 "Sky pixmaps",NULL,0,NONE);
	hr_update_button(optionbutton[34],UPDATE_HELPTEXT,
						  "Use precalced images instead of color slides?");
   optionbutton[35]=hr_create_button(182,291,317,302,
												 hr.font8x8,hr.def_colors,
												 "ExploFlash(tm)",NULL,0,NONE);
	hr_update_button(optionbutton[35],UPDATE_HELPTEXT,
						  "Flash color 0 when an explosion occurs?");
   optionbutton[36]=hr_create_button(182,304,317,315,
												 hr.font8x8,hr.def_colors,
												 "ExploColRot(tm)",NULL,0,NONE);
	hr_update_button(optionbutton[36],UPDATE_HELPTEXT,
						  "Rotate explosion colors?");
   optionbutton[37]=hr_create_button(182,317,317,328,
												 hr.font8x8,hr.def_colors,
												 "Shopping",NULL,0,NONE);
	hr_update_button(optionbutton[37],UPDATE_HELPTEXT,
						  "If this is disabled, you cannot buy more equipment.");
   optionbutton[43]=hr_create_button(182,330,317,341,
												 hr.font8x8,hr.def_colors,
												 "Nozzle flame",NULL,0,NONE);
	hr_update_button(optionbutton[43],UPDATE_HELPTEXT,
						  "Draw nozzle flames when weapons are fired?");
	/* Wide buttons */
	hr_bevelbox(0,400,319,479,hr.def_colors,NONE);
	optionbutton[38]=hr_create_button(3,402,317,413,
												hr.font8x8,hr.def_colors,
												"Reset to ~ud~uefaults",NULL,'d',
												BUTTON_ENABLED);
	optionbutton[39]=hr_create_button(3,415,317,426,
												hr.font8x8,hr.def_colors,
												"~uS~uave roster",NULL,'s',NONE);
	optionbutton[40]=hr_create_button(3,428,317,439,
												hr.font8x8,hr.def_colors,
												"~uH~uall of Fame",NULL,'h',BUTTON_ENABLED);
	optionbutton[41]=hr.default_button=
	  hr_create_button(3,441,317,452,
							 hr.font8x8,hr.def_colors,
							 "Back to ~ui~untro",NULL,'i',BUTTON_ENABLED);
	optionbutton[42]=hr_create_button(3,454,317,477,
												 hr.font8x8,hr.def_colors,
												 "Begin ~ug~uame","Let's Go!",'g',NONE);
}


/* Updates parts of the option screen */
void update_option_screen(int flags)
{
	int r,rc,bflags;
	if(flags & OPTION_ROSTER)
	  for(r=0;r<ROSTER_SIZE;r++) {
		  bflags=(((roster[r].flags & BUTTON_PRESSED)?(BUTTON_PRESSED):(NONE))
					 |BUTTON_ENABLED|BUTTON_SWITCH|BUTTON_NO_TAB_CYCLE);
		  hr_update_button(namebutton[r],UPDATE_FLAGS,&bflags);
	  }
	if(flags & OPTION_ONOFF) {
		/* Tracers */
		bflags=(((game_flags & TRACERS)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[22],UPDATE_FLAGS,&bflags);
#if 0
		/* Disasters */
		bflags=(((game_flags & DISASTERS)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[23],UPDATE_FLAGS,&bflags);
#endif
		/* Particles*/
		bflags=(((game_flags & PARTICLES)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[24],UPDATE_FLAGS,&bflags);
#ifdef SOUND_SUPPORT
		/* Sound effects*/
		if(hr.flags & HR_SOUND_INITIALIZED)
		  bflags=(((game_flags & SOUNDEFFECTS)?(BUTTON_PRESSED):(NONE))
					 |BUTTON_ENABLED|BUTTON_SWITCH);
		else
		  bflags=NONE;
		hr_update_button(optionbutton[25],UPDATE_FLAGS,&bflags);
#endif
		/* Unsafe Storage*/
		bflags=(((game_flags & ACCIDENTAL_FIRE)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[26],UPDATE_FLAGS,&bflags);
		/* Suicide penalty*/
		bflags=(((game_flags & SUICIDE_PENALTY)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[27],UPDATE_FLAGS,&bflags);
		/* AimAid(tm) */
		bflags=(((game_flags & AIM_AID)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[28],UPDATE_FLAGS,&bflags);
		/* Alien terrain */
		bflags=BUTTON_SWITCH;
		if(!(game_flags & TERRAIN_PIXMAPS))
		  bflags |=BUTTON_ENABLED;
		if(game_flags & ALIEN_TERRAIN)
		  bflags |=BUTTON_PRESSED;
		hr_update_button(optionbutton[29],UPDATE_FLAGS,&bflags);
		/* Alien sky*/
		bflags=BUTTON_SWITCH;
		if(!(game_flags & SKY_PIXMAPS))
		  bflags |=BUTTON_ENABLED;
		if(game_flags & ALIEN_SKY)
		  bflags |=BUTTON_PRESSED;
		hr_update_button(optionbutton[30],UPDATE_FLAGS,&bflags);
		/* Random order */
		bflags=(((game_flags & RANDOM_ORDER)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[31],UPDATE_FLAGS,&bflags);
		/* GroundSmoothing button is BUTTON_ENABLED only in non-zero-gravity */
		bflags=(((game_flags & GROUNDSMOOTHING)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_SWITCH);
		if(g>0)
		  bflags|=BUTTON_ENABLED;
		hr_update_button(optionbutton[32],UPDATE_FLAGS,&bflags);
		/* Terrain pixmaps*/
		bflags=(((game_flags & TERRAIN_PIXMAPS)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[33],UPDATE_FLAGS,&bflags);
		/* Sky pixmaps */
		bflags=(((game_flags & SKY_PIXMAPS)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[34],UPDATE_FLAGS,&bflags);
		/* ExploFlash*/
		bflags=(((game_flags & EXPLO_FLASH)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[35],UPDATE_FLAGS,&bflags);
		/* Explosion color rotation*/
		bflags=(((game_flags & EXPLO_COLROT)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[36],UPDATE_FLAGS,&bflags);
		/* Shopping */
		bflags=(((game_flags & SHOPPING)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[37],UPDATE_FLAGS,&bflags);
		/* Nozzleflame */
		bflags=(((game_flags & NOZZLEFLAME)?(BUTTON_PRESSED):(NONE))
				  |BUTTON_ENABLED|BUTTON_SWITCH);
		hr_update_button(optionbutton[43],UPDATE_FLAGS,&bflags);
	}
	if(flags & OPTION_BEGIN) {
		rc=0;
		for(r=0;r<ROSTER_SIZE;r++){
			if(roster[r].flags & BUTTON_PRESSED)
			  rc++;
		}
		
		bflags=(((rc+droids>1)&&
					((rc+droids)<(S_WIDTH/
									  (2*MIN_DISTANCE)))?(BUTTON_ENABLED):(NONE)));
		hr_update_button(optionbutton[42],UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_WIDE) {
		/* Save Roster */
		bflags=((game_flags & ROSTER_SAVED)?(NONE):(BUTTON_ENABLED));
		hr_update_button(optionbutton[39],UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_ROUNDS) {
		/* Rounds */
		sprintf(buffer,"%d",round_n);
		hr_textbox(101,122,162,135,
					  hr.font8x8,hr.def_colors,
					  buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		
		bflags=((round_n>1)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[0],UPDATE_FLAGS,&bflags);
		
		bflags=((round_n<MAX_ROUNDS)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[1],UPDATE_FLAGS,&bflags);
	}

	if(flags & OPTION_MONEY) {
		/* Money */
		bflags=((start_money>0)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[2],UPDATE_FLAGS,&bflags);
	   sprintf(buffer,"%d",start_money);
	   hr_textbox(101,140,162,153,
					  hr.font8x8,
					  hr.def_colors,buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		bflags=((start_money<MAX_MONEY)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[3],UPDATE_FLAGS,&bflags);
		/* WMoney */
		bflags=((winning_money>0)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[4],UPDATE_FLAGS,&bflags);
	   sprintf(buffer,"%d",winning_money);
	   hr_textbox(101,155,162,168,
					  hr.font8x8,
					  hr.def_colors,buffer,
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		bflags=((winning_money<MAX_WMONEY)?
				  (BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[5],UPDATE_FLAGS,&bflags);
		/* Interest */
		bflags=((interest>0)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[6],UPDATE_FLAGS,&bflags);
	   sprintf(buffer,"%d",interest);
	   hr_textbox(101,170,162,183,
					  hr.font8x8,
					  hr.def_colors,buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		bflags=((interest<MAX_INTEREST)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[7],UPDATE_FLAGS,&bflags);
	}
	
	if(flags & OPTION_MISC) {
		/* Gravity */
		bflags=((g>0)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[8],UPDATE_FLAGS,&bflags);
	   hr_textbox(101,188,162,201,
					  hr.font8x8,
					  hr.def_colors,gravity_string(buffer),
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		bflags=((g<MAX_G)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[9],UPDATE_FLAGS,&bflags);
		/* Wind */
		bflags=((windmode>0)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[10],UPDATE_FLAGS,&bflags);
	   hr_textbox(101,203,162,216,
					  hr.font8x8,
					  hr.def_colors,
					  windtext[windmode],STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		bflags=((windmode<MAX_WINDMODE)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[11],UPDATE_FLAGS,&bflags);
		/* Edges */
		bflags=((edges>0)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[12],UPDATE_FLAGS,&bflags);
		hr_textbox(101,218,162,231,
					  hr.font8x8,hr.def_colors,
					  edgetext[edges],STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);

		bflags=((edges<MAX_EDGEMODE)?(BUTTON_ENABLED):(NONE))|BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[13],UPDATE_FLAGS,&bflags);
		/* Steepness */
		bflags=BUTTON_NO_TAB_CYCLE;
		if(!(game_flags & TERRAIN_PIXMAPS))
		  bflags|=((bumpiness>0)?(BUTTON_ENABLED):(NONE));
		hr_update_button(optionbutton[14],UPDATE_FLAGS,&bflags);
		hr_textbox(101,233,162,246,
					  hr.font8x8,hr.def_colors,
					  hr_percentage_string(buffer,
												  0,MAX_BUMPINESS,DEF_BUMPINESS,
												  bumpiness,
												  "Min.","Max.","Normal"),
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|((game_flags & TERRAIN_PIXMAPS)?(NONE):(BUTTON_ENABLED)));
		bflags=BUTTON_NO_TAB_CYCLE;
		if(!(game_flags & TERRAIN_PIXMAPS))
		  bflags|=((bumpiness<MAX_BUMPINESS)?(BUTTON_ENABLED):(NONE));
		hr_update_button(optionbutton[15],UPDATE_FLAGS,&bflags);
		/* Air Resistance */
		bflags=BUTTON_NO_TAB_CYCLE;
		bflags|=((air_resistance>0)?(BUTTON_ENABLED):(NONE));
		hr_update_button(optionbutton[16],UPDATE_FLAGS,&bflags);
		hr_textbox(101,248,162,261,
					  hr.font8x8,hr.def_colors,
					  hr_percentage_string(buffer,
												  0,MAX_AIR_RESISTANCE,DEF_AIR_RESISTANCE,
												  air_resistance,
												  "None","Max.","Normal"),
					  STRING_START,STRING_END,
					  BUTTON_PRESSED|BUTTON_ENABLED);
		bflags=BUTTON_NO_TAB_CYCLE;
		bflags|=((air_resistance<MAX_AIR_RESISTANCE)?(BUTTON_ENABLED):(NONE));
		hr_update_button(optionbutton[17],UPDATE_FLAGS,&bflags);
	}
	
	/* FIX! */
	if(flags & OPTION_DROIDS) {
		/* Droid_n */
		bflags=/*((droids>0)?(BUTTON_ENABLED):(NONE))|*/BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[18],UPDATE_FLAGS,&bflags);
		sprintf(buffer,"%d",droids);
		hr_textbox(101,368,162,381,
					  hr.font8x8,hr.def_colors,
					  buffer,STRING_START,STRING_END,
					  BUTTON_PRESSED/*|BUTTON_ENABLED*/);
		bflags=/*((droids<MAX_DROIDS)?(BUTTON_ENABLED):(NONE))|*/BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[19],UPDATE_FLAGS,&bflags);
		/* Droid_sq */
		bflags=/*((droid_sq>0)?(BUTTON_ENABLED):(NONE))|*/BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[20],UPDATE_FLAGS,&bflags);
		hr_textbox(101,383,162,396,
					  hr.font8x8,hr.def_colors,
					  hr_percentage_string(buffer,
												  0,MAX_DROID_SQ,DEF_DROID_SQ,
												  droid_sq,
												  "AOLer",NULL,"Normal"),
					  STRING_START,STRING_END,
					  BUTTON_PRESSED/*|BUTTON_ENABLED*/);
		bflags=/*((droid_sq<MAX_DROID_SQ)?(BUTTON_ENABLED):(NONE))|*/
		  BUTTON_NO_TAB_CYCLE;
		hr_update_button(optionbutton[21],UPDATE_FLAGS,&bflags);
	}
}


/* This screen lets you change player names, etc etc... */
int option_screen()
{
	struct hr_button *ob,*b;
	int begin_game=0,back_to_intro=0,see_hall_of_fame=0,quit=0,r;
	
   hr_screen_open(G640x480x256,NONE);
	hr_default_palette();
	init_options_effect();
	create_option_screen();	
	update_option_screen(OPTION_ROSTER|OPTION_BEGIN|OPTION_ONOFF|
								OPTION_ROUNDS|OPTION_MONEY|OPTION_MISC|
								OPTION_DROIDS|OPTION_WIDE);
	/* Loop until "Begin game"-button is selected. */
	hr_input_handler(NULL,&options_effect,AMB_RELEASED);
	hr_clearkey();

	while(!begin_game && !back_to_intro &&
			!see_hall_of_fame && !quit){
		ob=hr_input_handler(NULL,&options_effect,LMB_PRESSED|KBD_PRESSED);
		/* Check special keypresses */
		switch(hr_keyboard.lastkey){
		 case 27:
			if(hr_ask_confirmation("Really quit?")){
				if(!(game_flags & ROSTER_SAVED)){
					if(hr_ask_confirmation("Save roster?"))
					  save_roster();
				}
				quit=1;
			}
			break;
		}
		b=hr_input_handler(ob,&options_effect,LMB_RELEASED);
		/* Buttons that respond after the mouse key is released.*/
		if(b!=NULL){
			/* Roster buttons */
			for(r=0;r<ROSTER_SIZE;r++){
				/* Info */
				if(infobutton[r]==b){
					if(option_quickinfo(r))
					  hr_update_button(namebutton[r],UPDATE_TEXT,roster[r].name);
					break;
				}
				/* Name */
				else if(namebutton[r]==b){
					/* Set the PRESSED flag in roster entry.*/
					roster[r].flags ^= BUTTON_PRESSED;
					update_option_screen(OPTION_BEGIN);
					break;
				}
			}
			for(r=0;r<MAX_OPTIONBUTTONS;r++){
				if(optionbutton[r]==b){
					switch(r) {			
					 case 0:
						--round_n;
						update_option_screen(OPTION_ROUNDS);
						break;
					 case 1:
						++round_n;
						update_option_screen(OPTION_ROUNDS);
						break;
					 case 2:
						start_money-=1000;
						update_option_screen(OPTION_MONEY);
						break;
					 case 3:
						start_money+=1000;
						update_option_screen(OPTION_MONEY);
						break;
					 case 4:
						winning_money-=1000;
						update_option_screen(OPTION_MONEY);
						break;
					 case 5:
						winning_money+=1000;
						update_option_screen(OPTION_MONEY);
						break;
					 case 6:
						interest-=5;
						update_option_screen(OPTION_MONEY);
						break;
					 case 7:
						interest+=5;
						update_option_screen(OPTION_MONEY);
						break;
					 case 8:
						--g;
						update_option_screen(OPTION_MISC|OPTION_ONOFF);
						break;
					 case 9:
						++g;
						update_option_screen(OPTION_MISC|OPTION_ONOFF);
						break;
					 case 10:
						--windmode;
						update_option_screen(OPTION_MISC);
						break;
					 case 11:
						++windmode;
						update_option_screen(OPTION_MISC);
						break;
					 case 12:
						--edges;
						update_option_screen(OPTION_MISC);
						break;
					 case 13:
						++edges;
						update_option_screen(OPTION_MISC);
						break;
					 case 14:
						--bumpiness;
						update_option_screen(OPTION_MISC);
						break;
					 case 15:
						++bumpiness;
						update_option_screen(OPTION_MISC);
						break;
					 case 16:
						--air_resistance;
						update_option_screen(OPTION_MISC);
						break;
					 case 17:
						++air_resistance;
						update_option_screen(OPTION_MISC);
						break;
					 case 18:
						--droids;
						update_option_screen(OPTION_DROIDS|OPTION_BEGIN);
						break;
					 case 19:
						++droids;
						update_option_screen(OPTION_DROIDS|OPTION_BEGIN);
						break;
					 case 20:
						--droid_sq;
						update_option_screen(OPTION_DROIDS);
						break;
					 case 21:
						++droid_sq;
						update_option_screen(OPTION_DROIDS);
						break;
					 case 22:
						game_flags ^= TRACERS;
						break;
#if 0
					 case 23:
						game_flags ^= DISASTERS;
						break;
#endif
					 case 24:
						game_flags ^= PARTICLES;
						break;
#ifdef SOUND_SUPPORT
					 case 25:
						game_flags ^= SOUNDEFFECTS;
						break;
#endif
					 case 26:
						game_flags ^= ACCIDENTAL_FIRE;
						break;
					 case 27:
						game_flags ^= SUICIDE_PENALTY;
						break;
					 case 28:
						game_flags ^= AIM_AID;
						break;
					 case 29:
						game_flags ^= ALIEN_TERRAIN;
						break;
					 case 30:
						game_flags ^= ALIEN_SKY;
						break;
					 case 31:
						game_flags ^= RANDOM_ORDER;
						break;
					 case 32:
						game_flags ^= GROUNDSMOOTHING;
						break;
					 case 33:
						game_flags ^= TERRAIN_PIXMAPS;
						update_option_screen(OPTION_ONOFF|OPTION_MISC);
						break;
					 case 34:
						game_flags ^= SKY_PIXMAPS;
						update_option_screen(OPTION_ONOFF|OPTION_MISC);
						break;
					 case 35:
						game_flags ^= EXPLO_FLASH;
						update_option_screen(OPTION_ONOFF);
						break;
					 case 36:
						game_flags ^= EXPLO_COLROT;
						update_option_screen(OPTION_ONOFF);
						break;
					 case 37:
						game_flags ^= SHOPPING;
						update_option_screen(OPTION_ONOFF);
						break;
					 case 43:
						game_flags ^= NOZZLEFLAME;
						update_option_screen(OPTION_ONOFF);
						break;
					 case 38:
						defaults();
						update_option_screen(OPTION_ONOFF|OPTION_WIDE|OPTION_ROUNDS|
													OPTION_MONEY|OPTION_MISC|OPTION_DROIDS);
						break;
					 case 39:
						save_roster();
						update_option_screen(OPTION_WIDE);
						break;
					 case 40:
						see_hall_of_fame=1;
						break;
					 case 41:
						back_to_intro=1;
						break;
					 case 42:	
						begin_game=1;
						break;	
					}
					break;
				}
			}
		}
	}
	/* Free some space.*/
	while(hr_delete_button(hr.firstbutton));
	free(option_stars);
	hr_screen_close(hr.active_screen);
	free(infobutton);
	free(namebutton);
	free(optionbutton);
	/**/
	if(quit)
	  return 3;
	else if(see_hall_of_fame)
	  return 2;
	else if(back_to_intro)
	  return 1;
	else
	  return 0;
}
